/*********************************************************************
 *
 *        Filename:  mtimer_drv.h
 *
 *         Created:  2022-04-28 02:19:34 PM
 *   Last Modified:  2022-04-28 02:19:34 PM
 *          Author:  xinxin.li
 *    Organization:  Beijing Smart Logic Technology Co., Ltd.
 *
 *     Description:
 *
 *
 ********************************************************************/

#ifndef __UCP_MTIMER_H__
#define __UCP_MTIMER_H__

#include "ucp_cpri.h"
#include "jesd204.h"

#define CPRI_TMR_BASE            0x08550000 			// CPRI_AUX_BASE + 0x0C000
#define ECPRI_TMR_BASE           0x08352000
#define JS_RX0_TMR_BASE          JS_COMC_BASE + 0x6000	// 0x06000000 + 0x6000
#define JS_RX1_TMR_BASE          JS_COMC_BASE + 0x7000
#define JS_TX0_TMR_BASE          JS_COMC_BASE + 0x8000
#define JS_TX1_TMR_BASE          JS_COMC_BASE + 0x9000

#define MTMR_PIN_CTRL_REG        (4*0x00)
#define MTMR_IO_CTRL_REG         (4*0x01)
#define MTMR_INTC_REG        (4*0x02)
#define MTMR_SCR_CTRL_REG    (4*0x03)
#define MTMR_CTRL_REG        (4*0x04)
#define MTMR_RX_SCRL_0_REG   (4*0x05)
#define MTMR_RX_SCRH_0_REG   (4*0x06)
#define MTMR_TX_SCRL_0_REG   (4*0x07)
#define MTMR_TX_SCRH_0_REG   (4*0x08)
#define MTMR_RX_SCRL_1_REG   (4*0x09)
#define MTMR_RX_SCRH_1_REG   (4*0x0A)
#define MTMR_TX_SCRL_1_REG   (4*0x0B)
#define MTMR_TX_SCRH_1_REG   (4*0x0C)
#define MTMR_TEVENT0_REG     (4*0x10)
#define MTMR_TEVENT1_REG     (4*0x11)
#define MTMR_TEVENT2_REG     (4*0x12)
#define MTMR_TSCR_TRG_REG    (4*0x13)
#define MTMR_TWREQ0_REG      (4*0x14)
#define MTMR_TWREQ1_REG      (4*0x15)
#define MTMR_TWREQ2_REG      (4*0x16)
#define MTMR_TINTE00_REG     (4*0x17)
#define MTMR_TINTF00_REG     (4*0x18)
#define MTMR_TINTE01_REG     (4*0x19)
#define MTMR_TINTF01_REG     (4*0x1A)
#define MTMR_TINTE02_REG     (4*0x1B)
#define MTMR_TINTF02_REG     (4*0x1C)
#define MTMR_TINTE10_REG     (4*0x1D)
#define MTMR_TINTF10_REG     (4*0x1E)
#define MTMR_TINTE11_REG     (4*0x1F)
#define MTMR_TINTF11_REG     (4*0x20)
#define MTMR_TINTE12_REG     (4*0x21)
#define MTMR_TINTF12_REG     (4*0x22)
#define MTMR_TINTE20_REG     (4*0x23)
#define MTMR_TINTF20_REG     (4*0x24)
#define MTMR_TINTE21_REG     (4*0x25)
#define MTMR_TINTF21_REG     (4*0x26)
#define MTMR_TINTE22_REG     (4*0x27)
#define MTMR_TINTF22_REG     (4*0x28)
#define MTMR_TINTE30_REG     (4*0x29)
#define MTMR_TINTF30_REG     (4*0x2A)
#define MTMR_TINTE31_REG     (4*0x2B)
#define MTMR_TINTF31_REG     (4*0x2C)
#define MTMR_TINTE32_REG     (4*0x2D)
#define MTMR_TINTF32_REG     (4*0x2E)
#define MTMR_TINTE40_REG     (4*0x2F)
#define MTMR_TINTF40_REG     (4*0x30)
#define MTMR_TINTE41_REG     (4*0x31)
#define MTMR_TINTF41_REG     (4*0x32)
#define MTMR_TINTE42_REG     (4*0x33)
#define MTMR_TINTF42_REG     (4*0x34)
#define MTMR_TINTE50_REG     (4*0x35)
#define MTMR_TINTF50_REG     (4*0x36)
#define MTMR_TINTE51_REG     (4*0x37)
#define MTMR_TINTF51_REG     (4*0x38)
#define MTMR_TINTE52_REG     (4*0x39)
#define MTMR_TINTF52_REG     (4*0x3A)
#define MTMR_TINTE60_REG     (4*0x3B)
#define MTMR_TINTF60_REG     (4*0x3C)
#define MTMR_TINTE61_REG     (4*0x3D)
#define MTMR_TINTF61_REG     (4*0x3E)
#define MTMR_TINTE62_REG     (4*0x3F)
#define MTMR_TINTF62_REG     (4*0x40)
#define MTMR_TINTE70_REG     (4*0x41)
#define MTMR_TINTF70_REG     (4*0x42)
#define MTMR_TINTE71_REG     (4*0x43)
#define MTMR_TINTF71_REG     (4*0x44)
#define MTMR_TINTE72_REG     (4*0x45)
#define MTMR_TINTF72_REG     (4*0x46)
#define MTMR_TINTE80_REG     (4*0x47)
#define MTMR_TINTF80_REG     (4*0x48)
#define MTMR_TINTE81_REG     (4*0x49)
#define MTMR_TINTF81_REG     (4*0x4A)
#define MTMR_TINTE82_REG     (4*0x4B)
#define MTMR_TINTF82_REG     (4*0x4C)
#define MTMR_TINTE90_REG     (4*0x4D)
#define MTMR_TINTF90_REG     (4*0x4E)
#define MTMR_TINTE91_REG     (4*0x4F)
#define MTMR_TINTF91_REG     (4*0x50)
#define MTMR_TINTE92_REG     (4*0x51)
#define MTMR_TINTF92_REG     (4*0x52)
#define MTMR_TINTEA0_REG     (4*0x53)
#define MTMR_TINTFA0_REG     (4*0x54)
#define MTMR_TINTEA1_REG     (4*0x55)
#define MTMR_TINTFA1_REG     (4*0x56)
#define MTMR_TINTEA2_REG     (4*0x57)
#define MTMR_TINTFA2_REG     (4*0x58)
#define MTMR_TINTEB0_REG     (4*0x59)
#define MTMR_TINTFB0_REG     (4*0x5A)
#define MTMR_TINTEB1_REG     (4*0x5B)
#define MTMR_TINTFB1_REG     (4*0x5C)
#define MTMR_TINTEB2_REG     (4*0x5D)
#define MTMR_TINTFB2_REG     (4*0x5E)
#define MTMR_CEVENT_REG      (4*0x5F)
#define MTMR_CMSK_REG        (4*0x60)
#define MTMR_CINTE0_REG      (4*0x61)
#define MTMR_CINTF0_REG      (4*0x62)
#define MTMR_CINTE1_REG      (4*0x63)
#define MTMR_CINTF1_REG      (4*0x64)
#define MTMR_CINTE2_REG      (4*0x65)
#define MTMR_CINTF2_REG      (4*0x66)
#define MTMR_CINTE3_REG      (4*0x67)
#define MTMR_CINTF3_REG      (4*0x68)
#define MTMR_CINTE4_REG      (4*0x69)
#define MTMR_CINTF4_REG      (4*0x6A)
#define MTMR_CINTE5_REG      (4*0x6B)
#define MTMR_CINTF5_REG      (4*0x6C)
#define MTMR_CINTE6_REG      (4*0x6D)
#define MTMR_CINTF6_REG      (4*0x6E)
#define MTMR_CINTE7_REG      (4*0x6F)
#define MTMR_CINTF7_REG      (4*0x70)
#define MTMR_CINTE8_REG      (4*0x71)
#define MTMR_CINTF8_REG      (4*0x72)
#define MTMR_CINTE9_REG      (4*0x73)
#define MTMR_CINTF9_REG      (4*0x74)
#define MTMR_CINTEA_REG      (4*0x75)
#define MTMR_CINTFA_REG      (4*0x76)
#define MTMR_CINTEB_REG      (4*0x77)
#define MTMR_CINTFB_REG      (4*0x78)
#define MTMR_OVFL_REG        (4*0x79)
#define MTMR_OVFH_REG        (4*0x7A)
#define MTMR_RXEN0_0L_REG    (4*0x7B)
#define MTMR_RXEN0_0H_REG    (4*0x7C)
#define MTMR_RXEN0_1L_REG    (4*0x7D)
#define MTMR_RXEN0_1H_REG    (4*0x7E)
#define MTMR_TXEN0_0L_REG    (4*0x7F)
#define MTMR_TXEN0_0H_REG    (4*0x80)
#define MTMR_TXEN0_1L_REG    (4*0x81)
#define MTMR_TXEN0_1H_REG    (4*0x82)
#define MTMR_RXEN1_0L_REG    (4*0x83)
#define MTMR_RXEN1_0H_REG    (4*0x84)
#define MTMR_RXEN1_1L_REG    (4*0x85)
#define MTMR_RXEN1_1H_REG    (4*0x86)
#define MTMR_TXEN1_0L_REG    (4*0x87)
#define MTMR_TXEN1_0H_REG    (4*0x88)
#define MTMR_TXEN1_1L_REG    (4*0x89)
#define MTMR_TXEN1_1H_REG    (4*0x8A)
#define MTMR_RXEN2CSU0L_REG  (4*0x8B)
#define MTMR_RXEN2CSU0H_REG  (4*0x8C)
#define MTMR_RXEN2CSU1L_REG  (4*0x8D)
#define MTMR_RXEN2CSU1H_REG  (4*0x8E)
#define MTMR_TXEN2CSU0L_REG  (4*0x8F)
#define MTMR_TXEN2CSU0H_REG  (4*0x90)
#define MTMR_TXEN2CSU1L_REG  (4*0x91)
#define MTMR_TXEN2CSU1H_REG  (4*0x92)
#define MTMR_SCRATH0L_REG    (4*0x93)
#define MTMR_SCRATH0H_REG    (4*0x94)
#define MTMR_SCRATH1L_REG    (4*0x95)
#define MTMR_SCRATH1H_REG    (4*0x96)
#define MTMR_RX_SCRL_REG     (4*0x97)
#define MTMR_RX_SCRH_REG     (4*0x98)
#define MTMR_TX_SCRL_REG     (4*0x99)
#define MTMR_TX_SCRH_REG     (4*0x9A)
#define MTMR_1PPSL_REG       (4*0x9B)
#define MTMR_1PPSH_REG       (4*0x9C)
#define MTMR_CPRI_10MS_CTRL  (4*0x9D)
#define MTMR_CPRI_ADJ_L      (4*0x9E)
#define MTMR_CPRI_ADJ_H      (4*0x9F)
#define MTMR_CPRI_10MS_CLR   (4*0xA0)

#define MTMR_RX_SOF0_SCR_L_REG  (4*0xA1)
#define MTMR_RX_SOF0_SCR_H_REG  (4*0xA2)
#define MTMR_RX_SOF1_SCR_L_REG  (4*0xA3)
#define MTMR_RX_SOF1_SCR_H_REG  (4*0xA4)
#define MTMR_RX_SOF2_SCR_L_REG  (4*0xA5)
#define MTMR_RX_SOF2_SCR_H_REG  (4*0xA6)
#define MTMR_RX_SOF3_SCR_L_REG  (4*0xA7)
#define MTMR_RX_SOF3_SCR_H_REG  (4*0xA8)
#define MTMR_RX_EOF0_SCR_L_REG  (4*0xA9)
#define MTMR_RX_EOF0_SCR_H_REG  (4*0xAA)
#define MTMR_RX_EOF1_SCR_L_REG  (4*0xAB)
#define MTMR_RX_EOF1_SCR_H_REG  (4*0xAC)
#define MTMR_RX_EOF2_SCR_L_REG  (4*0xAD)
#define MTMR_RX_EOF2_SCR_H_REG  (4*0xAE)
#define MTMR_RX_EOF3_SCR_L_REG  (4*0xAF)
#define MTMR_RX_EOF3_SCR_H_REG  (4*0xB0)
#define MTMR_RX_SOF_EOF_SCR_EN  (4*0xB1)


#define MTMR_TnL_REG0   	    (4*0x100+8*0x0)
#define MTMR_TnL_REG1   	    (4*0x100+8*0x1)
#define MTMR_TnL_REG2   	    (4*0x100+8*0x2)
#define MTMR_TnL_REG3   	    (4*0x100+8*0x3)
#define MTMR_TnL_REG4   	    (4*0x100+8*0x4)
#define MTMR_TnL_REG5   	    (4*0x100+8*0x5)
#define MTMR_TnL_REG6   	    (4*0x100+8*0x6)
#define MTMR_TnL_REG7   	    (4*0x100+8*0x7)
#define MTMR_TnL_REG8   	    (4*0x100+8*0x8)
#define MTMR_TnL_REG9   	    (4*0x100+8*0x9)
#define MTMR_TnL_REG10        (4*0x100+8*0xa)
#define MTMR_TnL_REG11        (4*0x100+8*0xb)
#define MTMR_TnL_REG12        (4*0x100+8*0xc)
#define MTMR_TnL_REG13        (4*0x100+8*0xd)
#define MTMR_TnL_REG14        (4*0x100+8*0xe)
#define MTMR_TnL_REG15        (4*0x100+8*0xf)
#define MTMR_TnL_REG16        (4*0x100+8*0x10)
#define MTMR_TnL_REG17        (4*0x100+8*0x11)
#define MTMR_TnL_REG18        (4*0x100+8*0x12)
#define MTMR_TnL_REG19        (4*0x100+8*0x13)
#define MTMR_TnL_REG20        (4*0x100+8*0x14)
#define MTMR_TnL_REG21        (4*0x100+8*0x15)
#define MTMR_TnL_REG22        (4*0x100+8*0x16)
#define MTMR_TnL_REG23        (4*0x100+8*0x17)
#define MTMR_TnL_REG24        (4*0x100+8*0x18)
#define MTMR_TnL_REG25        (4*0x100+8*0x19)
#define MTMR_TnL_REG26        (4*0x100+8*0x1a)
#define MTMR_TnL_REG27        (4*0x100+8*0x1b)
#define MTMR_TnL_REG28        (4*0x100+8*0x1c)
#define MTMR_TnL_REG29        (4*0x100+8*0x1d)
#define MTMR_TnL_REG30        (4*0x100+8*0x1e)
#define MTMR_TnL_REG31        (4*0x100+8*0x1f)
#define MTMR_TnL_REG32        (4*0x100+8*0x20)
#define MTMR_TnL_REG33        (4*0x100+8*0x21)
#define MTMR_TnL_REG34        (4*0x100+8*0x22)
#define MTMR_TnL_REG35        (4*0x100+8*0x23)
#define MTMR_TnL_REG36        (4*0x100+8*0x24)
#define MTMR_TnL_REG37        (4*0x100+8*0x25)
#define MTMR_TnL_REG38        (4*0x100+8*0x26)
#define MTMR_TnL_REG39        (4*0x100+8*0x27)
#define MTMR_TnL_REG40        (4*0x100+8*0x28)
#define MTMR_TnL_REG41        (4*0x100+8*0x29)
#define MTMR_TnL_REG42        (4*0x100+8*0x2a)
#define MTMR_TnL_REG43        (4*0x100+8*0x2b)
#define MTMR_TnL_REG44        (4*0x100+8*0x2c)
#define MTMR_TnL_REG45        (4*0x100+8*0x2d)
#define MTMR_TnL_REG46        (4*0x100+8*0x2e)
#define MTMR_TnL_REG47        (4*0x100+8*0x2f)
#define MTMR_TnL_REG48        (4*0x100+8*0x30)
#define MTMR_TnL_REG49        (4*0x100+8*0x31)
#define MTMR_TnL_REG50        (4*0x100+8*0x32)
#define MTMR_TnL_REG51        (4*0x100+8*0x33)
#define MTMR_TnL_REG52        (4*0x100+8*0x34)
#define MTMR_TnL_REG53        (4*0x100+8*0x35)
#define MTMR_TnL_REG54        (4*0x100+8*0x36)
#define MTMR_TnL_REG55        (4*0x100+8*0x37)
#define MTMR_TnL_REG56        (4*0x100+8*0x38)
#define MTMR_TnL_REG57        (4*0x100+8*0x39)
#define MTMR_TnL_REG58        (4*0x100+8*0x3a)
#define MTMR_TnL_REG59        (4*0x100+8*0x3b)
#define MTMR_TnL_REG60        (4*0x100+8*0x3c)
#define MTMR_TnL_REG61        (4*0x100+8*0x3d)
#define MTMR_TnL_REG62        (4*0x100+8*0x3e)
#define MTMR_TnL_REG63        (4*0x100+8*0x3f)
#define MTMR_TnL_REG64        (4*0x100+8*0x40)
#define MTMR_TnL_REG65        (4*0x100+8*0x41)
#define MTMR_TnL_REG66        (4*0x100+8*0x42)
#define MTMR_TnL_REG67        (4*0x100+8*0x43)
#define MTMR_TnL_REG68        (4*0x100+8*0x44)
#define MTMR_TnL_REG69        (4*0x100+8*0x45)
#define MTMR_TnL_REG70        (4*0x100+8*0x46)
#define MTMR_TnL_REG71        (4*0x100+8*0x47)
#define MTMR_TnL_REG72        (4*0x100+8*0x48)
#define MTMR_TnL_REG73        (4*0x100+8*0x49)
#define MTMR_TnL_REG74        (4*0x100+8*0x4a)
#define MTMR_TnL_REG75        (4*0x100+8*0x4b)
#define MTMR_TnL_REG76        (4*0x100+8*0x4c)
#define MTMR_TnL_REG77        (4*0x100+8*0x4d)
#define MTMR_TnL_REG78        (4*0x100+8*0x4e)
#define MTMR_TnL_REG79        (4*0x100+8*0x4f)
#define MTMR_TnL_REG80        (4*0x100+8*0x50)
#define MTMR_TnL_REG81        (4*0x100+8*0x51)
#define MTMR_TnL_REG82        (4*0x100+8*0x52)
#define MTMR_TnL_REG83        (4*0x100+8*0x53)
#define MTMR_TnL_REG84        (4*0x100+8*0x54)
#define MTMR_TnL_REG85        (4*0x100+8*0x55)
#define MTMR_TnL_REG86        (4*0x100+8*0x56)
#define MTMR_TnL_REG87        (4*0x100+8*0x57)
#define MTMR_TnL_REG88        (4*0x100+8*0x58)
#define MTMR_TnL_REG89        (4*0x100+8*0x59)
#define MTMR_TnL_REG90        (4*0x100+8*0x5a)
#define MTMR_TnL_REG91        (4*0x100+8*0x5b)
#define MTMR_TnL_REG92        (4*0x100+8*0x5c)
#define MTMR_TnL_REG93        (4*0x100+8*0x5d)
#define MTMR_TnL_REG94        (4*0x100+8*0x5e)
#define MTMR_TnL_REG95        (4*0x100+8*0x5f)



#define MTMR_TnH_REG0   	 (4*0x100+8*0x0 + 0x4)
#define MTMR_TnH_REG1   	 (4*0x100+8*0x1 + 0x4)
#define MTMR_TnH_REG2   	 (4*0x100+8*0x2 + 0x4)
#define MTMR_TnH_REG3   	 (4*0x100+8*0x3 + 0x4)
#define MTMR_TnH_REG4   	 (4*0x100+8*0x4 + 0x4)
#define MTMR_TnH_REG5   	 (4*0x100+8*0x5 + 0x4)
#define MTMR_TnH_REG6   	 (4*0x100+8*0x6 + 0x4)
#define MTMR_TnH_REG7   	 (4*0x100+8*0x7 + 0x4)
#define MTMR_TnH_REG8   	 (4*0x100+8*0x8 + 0x4)
#define MTMR_TnH_REG9   	 (4*0x100+8*0x9 + 0x4)
#define MTMR_TnH_REG10     (4*0x100+8*0xa + 0x4)
#define MTMR_TnH_REG11     (4*0x100+8*0xb + 0x4)
#define MTMR_TnH_REG12     (4*0x100+8*0xc + 0x4)
#define MTMR_TnH_REG13     (4*0x100+8*0xd + 0x4)
#define MTMR_TnH_REG14     (4*0x100+8*0xe + 0x4)
#define MTMR_TnH_REG15     (4*0x100+8*0xf + 0x4)
#define MTMR_TnH_REG16     (4*0x100+8*0x10 + 0x4)
#define MTMR_TnH_REG17     (4*0x100+8*0x11 + 0x4)
#define MTMR_TnH_REG18     (4*0x100+8*0x12 + 0x4)
#define MTMR_TnH_REG19     (4*0x100+8*0x13 + 0x4)
#define MTMR_TnH_REG20     (4*0x100+8*0x14 + 0x4)
#define MTMR_TnH_REG21     (4*0x100+8*0x15 + 0x4)
#define MTMR_TnH_REG22     (4*0x100+8*0x16 + 0x4)
#define MTMR_TnH_REG23     (4*0x100+8*0x17 + 0x4)
#define MTMR_TnH_REG24     (4*0x100+8*0x18 + 0x4)
#define MTMR_TnH_REG25     (4*0x100+8*0x19 + 0x4)
#define MTMR_TnH_REG26     (4*0x100+8*0x1a + 0x4)
#define MTMR_TnH_REG27     (4*0x100+8*0x1b + 0x4)
#define MTMR_TnH_REG28     (4*0x100+8*0x1c + 0x4)
#define MTMR_TnH_REG29     (4*0x100+8*0x1d + 0x4)
#define MTMR_TnH_REG30     (4*0x100+8*0x1e + 0x4)
#define MTMR_TnH_REG31     (4*0x100+8*0x1f + 0x4)
#define MTMR_TnH_REG32     (4*0x100+8*0x20 + 0x4)
#define MTMR_TnH_REG33     (4*0x100+8*0x21 + 0x4)
#define MTMR_TnH_REG34     (4*0x100+8*0x22 + 0x4)
#define MTMR_TnH_REG35     (4*0x100+8*0x23 + 0x4)
#define MTMR_TnH_REG36     (4*0x100+8*0x24 + 0x4)
#define MTMR_TnH_REG37     (4*0x100+8*0x25 + 0x4)
#define MTMR_TnH_REG38     (4*0x100+8*0x26 + 0x4)
#define MTMR_TnH_REG39     (4*0x100+8*0x27 + 0x4)
#define MTMR_TnH_REG40     (4*0x100+8*0x28 + 0x4)
#define MTMR_TnH_REG41     (4*0x100+8*0x29 + 0x4)
#define MTMR_TnH_REG42     (4*0x100+8*0x2a + 0x4)
#define MTMR_TnH_REG43     (4*0x100+8*0x2b + 0x4)
#define MTMR_TnH_REG44     (4*0x100+8*0x2c + 0x4)
#define MTMR_TnH_REG45     (4*0x100+8*0x2d + 0x4)
#define MTMR_TnH_REG46     (4*0x100+8*0x2e + 0x4)
#define MTMR_TnH_REG47     (4*0x100+8*0x2f + 0x4)
#define MTMR_TnH_REG48     (4*0x100+8*0x30 + 0x4)
#define MTMR_TnH_REG49     (4*0x100+8*0x31 + 0x4)
#define MTMR_TnH_REG50     (4*0x100+8*0x32 + 0x4)
#define MTMR_TnH_REG51     (4*0x100+8*0x33 + 0x4)
#define MTMR_TnH_REG52     (4*0x100+8*0x34 + 0x4)
#define MTMR_TnH_REG53     (4*0x100+8*0x35 + 0x4)
#define MTMR_TnH_REG54     (4*0x100+8*0x36 + 0x4)
#define MTMR_TnH_REG55     (4*0x100+8*0x37 + 0x4)
#define MTMR_TnH_REG56     (4*0x100+8*0x38 + 0x4)
#define MTMR_TnH_REG57     (4*0x100+8*0x39 + 0x4)
#define MTMR_TnH_REG58     (4*0x100+8*0x3a + 0x4)
#define MTMR_TnH_REG59     (4*0x100+8*0x3b + 0x4)
#define MTMR_TnH_REG60     (4*0x100+8*0x3c + 0x4)
#define MTMR_TnH_REG61     (4*0x100+8*0x3d + 0x4)
#define MTMR_TnH_REG62     (4*0x100+8*0x3e + 0x4)
#define MTMR_TnH_REG63     (4*0x100+8*0x3f + 0x4)
#define MTMR_TnH_REG64     (4*0x100+8*0x40 + 0x4)
#define MTMR_TnH_REG65     (4*0x100+8*0x41 + 0x4)
#define MTMR_TnH_REG66     (4*0x100+8*0x42 + 0x4)
#define MTMR_TnH_REG67     (4*0x100+8*0x43 + 0x4)
#define MTMR_TnH_REG68     (4*0x100+8*0x44 + 0x4)
#define MTMR_TnH_REG69     (4*0x100+8*0x45 + 0x4)
#define MTMR_TnH_REG70     (4*0x100+8*0x46 + 0x4)
#define MTMR_TnH_REG71     (4*0x100+8*0x47 + 0x4)
#define MTMR_TnH_REG72     (4*0x100+8*0x48 + 0x4)
#define MTMR_TnH_REG73     (4*0x100+8*0x49 + 0x4)
#define MTMR_TnH_REG74     (4*0x100+8*0x4a + 0x4)
#define MTMR_TnH_REG75     (4*0x100+8*0x4b + 0x4)
#define MTMR_TnH_REG76     (4*0x100+8*0x4c + 0x4)
#define MTMR_TnH_REG77     (4*0x100+8*0x4d + 0x4)
#define MTMR_TnH_REG78     (4*0x100+8*0x4e + 0x4)
#define MTMR_TnH_REG79     (4*0x100+8*0x4f + 0x4)
#define MTMR_TnH_REG80     (4*0x100+8*0x50 + 0x4)
#define MTMR_TnH_REG81     (4*0x100+8*0x51 + 0x4)
#define MTMR_TnH_REG82     (4*0x100+8*0x52 + 0x4)
#define MTMR_TnH_REG83     (4*0x100+8*0x53 + 0x4)
#define MTMR_TnH_REG84     (4*0x100+8*0x54 + 0x4)
#define MTMR_TnH_REG85     (4*0x100+8*0x55 + 0x4)
#define MTMR_TnH_REG86     (4*0x100+8*0x56 + 0x4)
#define MTMR_TnH_REG87     (4*0x100+8*0x57 + 0x4)
#define MTMR_TnH_REG88     (4*0x100+8*0x58 + 0x4)
#define MTMR_TnH_REG89     (4*0x100+8*0x59 + 0x4)
#define MTMR_TnH_REG90     (4*0x100+8*0x5a + 0x4)
#define MTMR_TnH_REG91     (4*0x100+8*0x5b + 0x4)
#define MTMR_TnH_REG92     (4*0x100+8*0x5c + 0x4)
#define MTMR_TnH_REG93     (4*0x100+8*0x5d + 0x4)
#define MTMR_TnH_REG94     (4*0x100+8*0x5e + 0x4)
#define MTMR_TnH_REG95     (4*0x100+8*0x5f + 0x4)


#endif

